import argparse
import logging
import re

from aim import Run
from hydra import compose, initialize
from hydra.core.plugins import Plugins
from omegaconf import OmegaConf
from .command import Command
from ..utils.launcher import launch, launch_on_slurm
from ..utils.slurm import is_submitit_available
from ..utils.registry import Registry
from ..utils.hydra_plugin import UrartuPlugin

logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")


def register_my_plugin() -> None:
    Plugins.instance().register(UrartuPlugin)


register_my_plugin()


@Command.register("launch")
class Launch(Command):
    """
    Launches an action from a specific module
    """

    def add_subparser(self, parser: argparse._SubParsersAction) -> argparse.ArgumentParser:
        description = """urartu: launcher"""
        available_modules = ", ".join(Registry.load_file_content().keys())
        subparser = parser.add_parser(
            self.name,
            description=description,
            help=f"Launch an action from a given module: {available_modules}",
        )

        subparser.add_argument("--name", type=str, help="name of the project/module")
        subparser.add_argument("module_args", nargs=argparse.REMAINDER, help="module arguments")

        subparser.set_defaults(fire=self._launch)

        return subparser

    def _launch(self, args: argparse.Namespace):
        module_name = re.sub(r"[^A-Za-z0-9]+", "", args.name)
        module_path = Registry.get_module_path_by_name(module_name)

        with initialize(version_base=None, config_path="../config"):
            cfg = compose(config_name="main", overrides=args.module_args)
        cfg = OmegaConf.create(OmegaConf.to_container(cfg, resolve=True, enum_to_str=True))

        aim_run = Run(
            repo=cfg.aim.repo,
            experiment=cfg.action_config.experiment_name,
        )
        aim_run.set("cfg", cfg, strict=False)

        if cfg.slurm.use_slurm:
            assert is_submitit_available(), "Please 'pip install submitit' to schedule jobs on SLURM"

            launch_on_slurm(
                module=module_path,
                action_name=cfg.action_name,
                cfg=cfg,
                aim_run=aim_run,
            )
        else:
            launch(
                module=module_path,
                action_name=cfg.action_name,
                cfg=cfg,
                aim_run=aim_run,
            )

        if aim_run.active:
            aim_run.close()



import logging

import torch


class Device:
    DEVICE = None

    @staticmethod
    def set_device(device_name):
        if device_name == "auto":
            Device.DEVICE = "auto"
        elif device_name == "cuda":
            assert device_name == "cuda" and torch.cuda.is_available()
            Device.DEVICE = torch.device("cuda")
        else:
            Device.DEVICE = torch.device("cpu")

        logging.info(f"Using DEVICE: {Device.DEVICE}")

    @staticmethod
    def get_device():
        return Device.DEVICE



import argparse
import logging
import re

from .command import Command
from ..utils.registry import Registry


@Command.register("unregister")
class Unregister(Command):
    """
    Unregisters/deletes a module by given name.
    """

    def add_subparser(self, parser: argparse._SubParsersAction) -> argparse.ArgumentParser:
        description = """urartu: unregister"""
        subparser = parser.add_parser(
            self.name,
            description=description,
            help="Unregister a project from the registry",
        )

        subparser.add_argument("--name", type=str, help="name of the project/module")

        subparser.set_defaults(fire=self._unregister)

        return subparser

    def _unregister(self, args: argparse.Namespace):
        module_name = re.sub(r"[^A-Za-z0-9]+", "", args.name)

        if Registry.remove_entry(module_name):
            logging.info(f"Module '{module_name}' is unregistered successfully")



from hydra.core.config_search_path import ConfigSearchPath
from hydra.plugins.search_path_plugin import SearchPathPlugin

from urartu.utils.registry import Registry
from urartu.utils.user import get_current_user
import logging

logger = logging.getLogger(__name__)

current_user = get_current_user()
registry_paths = Registry.get_module_paths()

class UrartuPlugin(SearchPathPlugin):
    def manipulate_search_path(self, search_path: ConfigSearchPath) -> None:
        if registry_paths:
            for registry_path in registry_paths:
                paths = [
                    (f"{registry_path}/configs", "file"),
                    (f"{registry_path}/configs_{current_user}", "file"),
                    (f"pkg://{registry_path}/configs", "pkg"),
                    (f"file://{registry_path}/configs_{current_user}", "file")
                ]
                for path, type in paths:
                    search_path.prepend(provider="urartu", path=path)
                    logger.debug(f"Prepended {type} path: {path}")
        else:
            logger.warning("No registry paths found for UrartuPlugin.")